/**
 * [CSRD] SourceCBL Integration
 * 
 * An optimized version of the SourceCBL plugin.
 */
#pragma semicolon 1
#include <sourcemod>

#pragma newdecls required
#include "source_cbl/webapi.sp"

#define PLUGIN_VERSION "0.2.0"
public Plugin myinfo = {
	name = "[CSRD] SourceCBL Integration",
	author = "nosoop",
	description = "Personal integartion of the SourceCBL API.",
	version = PLUGIN_VERSION,
	url = "https://git.csrd.science/"
}

#define REQUESTER_CONSOLE -1
#define INVALID_USERID -1

char g_CommunityKickLog[PLATFORM_MAX_PATH];

public void OnPluginStart() {
	CreateConVar("sm_scbl_enabled", "1",
			"Teamwork.TF convar for A2S_RULES's sake.  Does absolutely nothing else.",
			FCVAR_NOTIFY);
	
	// really though, does this actually matter?  we're just going to skip the thing anyways
	int port = FindConVar("hostport").IntValue;
	char portString[6];
	IntToString(port, portString, sizeof(portString));
	
	
	RegAdminCmd("cbl_check_steamid", CheckCBLStatus, ADMFLAG_ROOT);
	
	BuildPath(Path_SM, g_CommunityKickLog, sizeof(g_CommunityKickLog), "logs/sourcecbl.log");
}

public void OnClientAuthorized(int client) {
	if (!IsFakeClient(client) && !SkipCommunityBanListCheck(client)) {
		RequestClientBanStatus(client);
	}
}

bool SkipCommunityBanListCheck(int client) {
	// TODO check if account is whitelisted (command group / keyvalues / clientprefs cache, etc.)
	return CheckCommandAccess(client, "cbl_whitelist", 0);
}

void RequestClientBanStatus(int client) {
	char steamid64[64];
	
	if (GetClientAuthId(client, AuthId_SteamID64, steamid64, sizeof(steamid64))) {
		QueryBanList(steamid64, OnSteamAuthBanStatusReceived, GetClientUserId(client));
	}
}

public Action CheckCBLStatus(int client, int argc) {
	char steamid64[64];
	GetCmdArgString(steamid64, sizeof(steamid64));
	StripQuotes(steamid64);
	
	// TODO make sure this is a valid steamid
	QueryBanList(steamid64, OnCommandBanStatusReceived,
			client? GetClientUserId(client) : REQUESTER_CONSOLE);
	
	return Plugin_Handled;
}

public void OnSteamAuthBanStatusReceived(const char[] steamid64, bool bBanned, int userid) {
	// player in banlist, request ban
	if (bBanned && userid != INVALID_USERID) {
		int client = GetClientOfUserId(userid);
		if (client) {
			KickClient(client,
					"You have been banned by SourceCBL for hacking / cheating."
					... "\nVisit www.SourceCBL.com for more information.");
			
			LogToFile(g_CommunityKickLog, "Kicked \"%L\" from server.", client);
		}
	}
}

public void OnCommandBanStatusReceived(const char[] steamid64, bool bBanned, int requester) {
	int client = requester == REQUESTER_CONSOLE? 0 : GetClientOfUserId(requester);
	
	if (requester == REQUESTER_CONSOLE || client) {
		ReplyToCommand(client, "Steam account %s CBL status: %b", steamid64, bBanned);
	}
}
